
/* Chrysalide - Outil d'analyse de fichiers binaires
 * dwarf.c - support du format DWARF v3
 *
 * Copyright (C) 2015 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "dwarf.h"


#include "../dwarf-int.h"



/* Format de débogage DWARF v3 (instance) */
struct _GDwarfV3Format
{
    GDwarfFormat parent;                    /* A laisser en premier        */

};

/* Format de débogage DWARF v3 (classe) */
struct _GDwarfV3FormatClass
{
    GDwarfFormatClass parent;               /* A laisser en premier        */

};


/* Initialise la classe des formats de débogage DWARF v3. */
static void g_dwarfv3_format_class_init(GDwarfV3FormatClass *);

/* Initialise une instance de format de débogage DWARF v3. */
static void g_dwarfv3_format_init(GDwarfV3Format *);

/* Supprime toutes les références externes. */
static void g_dwarfv3_format_dispose(GDwarfV3Format *);

/* Procède à la libération totale de la mémoire. */
static void g_dwarfv3_format_finalize(GDwarfV3Format *);



/* Indique le type défini pour un format de débogage DWARF v3. */
G_DEFINE_TYPE(GDwarfV3Format, g_dwarfv3_format, G_TYPE_DWARF_FORMAT);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des formats de débogage DWARF v3.       *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dwarfv3_format_class_init(GDwarfV3FormatClass *klass)
{
    GObjectClass *object;                   /* Autre version de la classe  */

    object = G_OBJECT_CLASS(klass);

    object->dispose = (GObjectFinalizeFunc/* ! */)g_dwarfv3_format_dispose;
    object->finalize = (GObjectFinalizeFunc)g_dwarfv3_format_finalize;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : format = instance à initialiser.                             *
*                                                                             *
*  Description : Initialise une instance de format de débogage DWARF v3.      *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dwarfv3_format_init(GDwarfV3Format *format)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : format = instance d'objet GLib à traiter.                    *
*                                                                             *
*  Description : Supprime toutes les références externes.                     *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dwarfv3_format_dispose(GDwarfV3Format *format)
{
    G_OBJECT_CLASS(g_dwarfv3_format_parent_class)->dispose(G_OBJECT(format));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : format = instance d'objet GLib à traiter.                    *
*                                                                             *
*  Description : Procède à la libération totale de la mémoire.                *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_dwarfv3_format_finalize(GDwarfV3Format *format)
{
    G_OBJECT_CLASS(g_dwarfv3_format_parent_class)->finalize(G_OBJECT(format));

}


/******************************************************************************
*                                                                             *
*  Paramètres  : content = contenu binaire à parcourir.                       *
*                parent  = éventuel format exécutable déjà chargé.            *
*                                                                             *
*  Description : Prend en charge un nouveau format DWARF (v3).                *
*                                                                             *
*  Retour      : Adresse de la structure mise en place ou NULL en cas d'échec.*
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GBinFormat *g_dwarfv3_format_new(GBinContent *content, GExeFormat *parent)
{
    GDwarfV3Format *result;                 /* Structure à retourner       */

    result = g_object_new(G_TYPE_DWARFV3_FORMAT, NULL);



    g_binary_format_set_content(G_BIN_FORMAT(result), content);



    return G_BIN_FORMAT(result);

}
